$ifndef _FIR_INC_
$define _FIR_INC_
'
' ------------------------------------------------------------------------------------------------------------
'| This source code is provided 'as is' and any express or implied warranties, including,                     |
'| but not limited to, the implied warranties of merchantability and fitness for a particular                 |
'| purpose are disclaimed.                                                                                    |
'|                                                                                                            |
'| In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,    |
'| special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute    |
'| goods or services; loss of use, data, or profits;  or business interruption) however caused and on any     |
'| theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)    |
'| arising in any way out of the use of this source code, even if advised of the possibility of such damage.  |
'|                                                                                                            |
'| Any views and conclusions contained in the source code and any documentation are those of the author's     |
'| and should not be interpreted as representing official policies,                                           |
'| either expressed or implied, of Crownhill Associates Ltd.                                                  |
' ------------------------------------------------------------------------------------------------------------
'
' FIR Filter procedures for Proton24
' Written by Les Johnson 10-05-14
'
' FIR filter structure access
'
    Symbol __oNumCoeffs  = 0                    ' FIRStruct_NumCoeffs
    Symbol __oCoeffsBase = 2                    ' FIRStruct_CoeffsBase
    Symbol __oDelayBase  = 4                    ' FIRStruct_DelayBase
    Symbol __oDelaySize  = 6                    ' FIRStruct_DelaySize

'---------------------------------------------------------------------------------------
' Initialise the FIR filter structure
' Input     : pFIRStruct (WREG10) = Address of the FIR filter structure
'           : pNumCoeffs (WREG11) = Number of coeffs in filter
'           : pCoeffsBase (WREG2) = Base address of filter coeffs either in PSV program memory
'           : pDelayBase (WREG3)  = Base address of delay buffer. Only in Y RAM
'           : pDelaySize (WREG4)  = ' The size of the delay buffer in Y RAM
' Output    : None
' Notes     : Creates the structure variables when the procedure is used in the BASIC program
'             FIRStruct_NumCoeffs = numCoeffs
'             FIRStruct_CoeffsBase = coeffsBase
'             FIRStruct_DelayBase = delayBase
'             FIRStruct_DelaySize = delayBaseSize
'
Proc FIR_StructInit(ByRef pFIRStruct As WREG10, ByRef pCoeffsBase As WREG2, pNumCoeffs As WREG11, ByRef pDelayBase As WREG3, pDelaySize As WREG4)
'
' Create the equivalent of a structure by using an array and aliasing
'
Global Dim FIRStruct[4] As Word                    ' Create a Word array to act as the structure holder
Global Dim FIRStruct_NumCoeffs  As FIRStruct_0     ' Number of coeffs in filter
Global Dim FIRStruct_CoeffsBase As FIRStruct_1     ' Base address of filter coeffs either in PSV program memory
Global Dim FIRStruct_DelayBase  As FIRStruct_2     ' Base address of delay buffer, only in Y RAM
Global Dim FIRStruct_DelaySize  As FIRStruct_3     ' The size of the delay buffer in Y RAM

    Ptr16(WREG10++) = pNumCoeffs
    Ptr16(WREG10++) = pCoeffsBase
    Ptr16(WREG10++) = pDelayBase
    Ptr16(WREG10++) = pDelaySize
EndProc

'---------------------------------------------------------------------------------------
' FIR (Finite Impulse Response) filter using a structure for its details
' Input     : pFIRStruct (WREG4) = Address of the Filter structure
'           : pIndex     (WREG3) = Index of the current sample
' Output    : WREG0 holds the current filter value
' Notes     : None
'
Proc FIR_Filter(ByRef pFIRStruct As WREG4, pIndex As WREG3), WREG0
    Push    CORCON
    Push    MODCON
    Mov.w   [W4 + __oNumCoeffs], W5     ' WREG5 now holds the amount of filter taps
    Mov.w   [W4 + __oCoeffsBase], W9    ' WREG9 now holds the address of the filter taps
    Mov.w   [W4 + __oDelaySize], W1     ' WREG1 now holds the buffer size
    Sl.w    W1, #1, W1
    Mov.w   [W4 + __oDelayBase], W2     ' WREG2 now holds the address of the input buffer
    Add.w   W1, W2, W1
    WREG0 = AddressOf(YMODEND)
    Sub.w   W1, #1, [W0]
    YMODSRT = WREG2                     ' WREG2 holds the address of the input buffer
    Sl.w    W3, #1, W1                  ' WREG3 holds the Current sample value
    Add.w   W1, W2, W10                 ' WREG2 holds the address of the input buffer
    CORCON = 244                        ' Prepare CORCON for fractional computation
    MODCON = $40A0                      ' Prepare MODCON for modulo operations
    Inc WREG5                           ' Increment the filter order value
    WREG4 = 0
    WREG6 = 0
    Clr     A
    mRepeat W5
        Mac     W4 * W6, A, [W9] += 2, W4, [W10] -= 2, W6
    Sac.r   A, #0, W0
    Pop     MODCON
    Pop     CORCON
EndProc

'---------------------------------------------------------------------------------------
' FIR (Finite Impulse Response) filter using parameters for its details
' Input     : pFilterTaps (WREG5) = The amount of taps (coefficients) in the filter
'           : pPtrCoeffs  (WREG9) = Address of the filter coefficients in PSV code memory
'           : pBuffLength (WREG4) = Amount of input samples
'           : pPtrInput   (WREG2) = Address of the input samples (Must be in Y RAM)
'           : pIndex      (WREG3) = Index of current sample
' Output    : WREG0 holds the current filter value
' Notes     : None
'
Proc FIR_Radix(pFilterTaps As WREG5, ByRef pPtrCoeffs As WREG9, pBuffLength As WREG4, ByRef pPtrInput As WREG2, pIndex As WREG3), WREG0
    Push    CORCON
    Push    MODCON
    Sl.w    W4, #1, W1                  ' WREG4 holds the buffer size
    Add.w   W1, W2, W1                  ' WREG2 holds the address of the input buffer
    WREG0 = AddressOf(YMODEND)
    Sub.w   W1, #1, [W0]
    YMODSRT = WREG2                     ' WREG2 holds the address of the input buffer
    Sl.w    W3, #1, W1                  ' WREG3 holds the Current sample value
    Add.w   W1, W2, W10                 ' WREG2 holds the address of the input buffer
    CORCON = 244                        ' Prepare CORCON for fractional computation
    MODCON = $40A0                      ' Prepare MODCON for modulo operations
    Inc WREG5                           ' Increment the filter order value
    WREG4 = 0
    WREG6 = 0
    Clr     A
    mRepeat W5
        Mac     W4 * W6, A, [W9] += 2, W4, [W10] -= 2, W6
    Sac.r   A, #0, W0
    Pop     MODCON
    Pop     CORCON
EndProc

$endif  ' _FIR_INC_
