'
' ------------------------------------------------------------------------------------------------------------
'| This source code is provided 'as is' and any express or implied warranties, including,                     |
'| but not limited to, the implied warranties of merchantability and fitness for a particular                 |
'| purpose are disclaimed.                                                                                    |
'|                                                                                                            |
'| In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,    |
'| special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute    |
'| goods or services; loss of use, data, or profits;  or business interruption) however caused and on any     |
'| theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)    |
'| arising in any way out of the use of this source code, even if advised of the possibility of such damage.  |
'|                                                                                                            |
'| Any views and conclusions contained in the source code and any documentation are those of the author's     |
'| and should not be interpreted as representing official policies,                                           |
'| either expressed or implied, of Crownhill Associates Ltd.                                                  |
' ------------------------------------------------------------------------------------------------------------
'
' 2.0 KHz FIR Lowpass Filter Example for Proton24
' Written by Les Johnson 10-05-14
'
' Place a signal on AN0 and the filtered signal appears on the left channel DAC
' An oscilloscope placed on the DAC will show the signal alter when the desired filter frequency/s are reached
'
' The signal on AN0 should be no more than 1.0 Volt and have a DC offset of 1.7 Volts.
'
    Device = 33FJ128GP802                                                   ' Choose a dsPIC33 device with a DAC peripheral
    Declare Xtal = 79.23                                                    ' Operate the dFevice at 79.23MHz

    Include "DAC.inc"                                                       ' Load the DAC procedures into the program
    Include "FIR.inc"                                                       ' Load the FIR procedures into the program
'
' Timer1 interrupt (ADC sample) rate calculator
'
    $define cSample_Rate 44100                                              ' Sample rate of the ADC (in Hz)
    $define cMicroSeconds $eval (1000000 / cSample_Rate)                    ' Interrupt rate (in us). Sample rate is (1000000 / Frequency (in Hz))
    $define cTweak_Value 36                                                 ' Added tweak value for rounding errors
    $define cPR1_Value $eval (cMicroSeconds * (_xtal / 2)) + cTweak_Value   ' Calculate the actual value to place into the PR1 SFR
'
' Do some error checking for the PR1 value
'
$if cPR1_Value > 65535
    $error "cPR1_Value too large for interrupt duration"
$elseif cPR1_Value = 0
    $error "cPR1_Value too small for interrupt duration"
$endif

    $define cBUFFER_SIZE 128                                                ' The size of the YRAM buffer
    $define cFILTER_TAPS 129                                                ' The amount of filter taps (the coefficient table)
'
' Coefficient table addressable in PSV space
' The values below are for a 2.0 KHz low pass filter using a Hamming window
'
    Dim cCOEFF As PSV = As Word $FFF4, $FFEE, $FFE8, $FFE4, $FFE1, $FFE1,_
                                $FFE3, $FFE8, $FFF0, $FFFB, $0009, $0019,_
                                $002A, $003B, $0049, $0053, $0057, $0053,_
                                $0046, $002F, $000F, $FFE7, $FFBB, $FF8D,_
                                $FF61, $FF3E, $FF26, $FF20, $FF2D, $FF51,_
                                $FF8B, $FFD9, $0038, $00A0, $0109, $016A,_
                                $01B8, $01EA, $01F5, $01D5, $0184, $0104,_
                                $0059, $FF8B, $FEA9, $FDC2, $FCEB, $FC39,_
                                $FBC2, $FB99, $FBD1, $FC76, $FD8F, $FF1C,_
                                $0117, $0370, $0612, $08E4, $0BC5, $0E93,_
                                $112B, $136D, $153C, $167E, $1723, $1723,_
                                $167E, $153C, $136D, $112B, $0E93, $0BC5,_
                                $08E4, $0612, $0370, $0117, $FF1C, $FD8F,_
                                $FC76, $FBD1, $FB99, $FBC2, $FC39, $FCEB,_
                                $FDC2, $FEA9, $FF8B, $0059, $0104, $0184,_
                                $01D5, $01F5, $01EA, $01B8, $016A, $0109,_
                                $00A0, $0038, $FFD9, $FF8B, $FF51, $FF2D,_
                                $FF20, $FF26, $FF3E, $FF61, $FF8D, $FFBB,_
                                $FFE7, $000F, $002F, $0046, $0053, $0057,_
                                $0053, $0049, $003B, $002A, $0019, $0009,_
                                $FFFB, $FFF0, $FFE8, $FFE3, $FFE1, $FFE1,_
                                $FFE4, $FFE8, $FFEE, $FFF4
'
' Variables used for the filter
'
    Dim Global_wFilterValue As SWord                    ' Final value from the filter
    Dim Global_wInIndex As Word                         ' Input buffer index
    Dim Global_wInputBuf[cBUFFER_SIZE] As Word YRAM     ' Input buffer, must be in Y RAM

'----------------------------------------------------------------------------------------------------
' Setup the ADC and initialise an ADC interrupt
' Input     : None
' Output    : None
' Notes     : The procedure also encompasses the interrupt handler
'
Proc ADC_Start()
    Input PORTA.0                           ' Make AN0 an input
    AD1CON2 = 0                             ' AVdd, AVss, MUXA only
    AD1PCFGLbits_PCFG0 = 0                  ' Analogue input on AN0
    AD1CHS0 = 0                             ' Configure AN0 as the ADC input
    AD1CON2 = $0000
    AD1CON3 = $021A                         ' Sampling time = 3 * Tad, minimum Tad selected
    AD1CON1 = %1000001111100010
'              ||||||||||||||||______________ Bit-0 : No ADC conversion yet
'              |||||||||||||||_______________ Bit-1 : ADC Sample and Hold amplifiers are holding
'              ||||||||||||||________________ Bit-2 : Sampling begins immediately after last conversion. SAMP bit is auto-set.
'              |||||||||||||_________________ Bit-3 : Samples multiple channels individually in sequence
'              ||||||||||||__________________ Bit-4 : Sample Clock Source Group Bit
'              |||||||||||___________________ Bit-5 :\
'              ||||||||||____________________ Bit-6 :| Internal Counter ends sampling and starts conversion (auto-convert)
'              |||||||||_____________________ Bit-7 :/
'              ||||||||______________________ Bit-8 :\ ADC is using signed fractional (DOUT = sddddddddddd)
'              |||||||_______________________ Bit-9 :/
'              ||||||________________________ Bit-10: 12-Bit, 1-channel ADC operation
'              |||||_________________________ Bit-11: Unimplemented
'              ||||__________________________ Bit-12: DMA buffers are written in Scatter/Gather mode
'              |||___________________________ Bit-13: Continue module operation in Idle mode
'              ||____________________________ Bit-14: Unimplemented
'              |_____________________________ Bit-15: Enable the ADC

    IEC0bits_AD1IE = 1                      ' ADC interrupt enabled
    AD1CON1bits_ADON = 1                    ' Enable the ADC
    AD1CON1bits_SAMP = 1                    ' Start a sample
'
'-------------------------------------------
' ADC interrupt handler
' Input     : None
' Output    : Global_wFilterValue holds the filtered value
' Notes     : Filters the ADC result
'
Isr ADC1Interrupt
    Global_wInputBuf[Global_wInIndex] = ADCBUF0             ' Get the ADC sample
'
' Filter the ADC samples into Global_wFilterValue
'
    Global_wFilterValue = FIR_Filter(FIRStruct, Global_wInIndex)
'
' Manage the filter's buffer
'
    Inc Global_wInIndex                                     ' Increment the samples buffer index
    If Global_wInIndex >= cBUFFER_SIZE Then                 ' Have we reached the end of the buffer?
        Global_wInIndex = 0                                 ' Yes. So reset it
    EndIf
    IFS0bits_AD1IF = 0                                      ' Clear the ADC interrupt flag
EndIsr
EndProc

'----------------------------------------------------------------------------------------------------
' Initialise a Timer1 interrupt
' Input     : None
' Output    : None
' Notes     : The procedure also encompasses the Interrupt handler
'
Proc Timer1_Start()
    T1CON = 0                                   ' Reset the T1CON SFR
    T1CONbits_TCKPS0 = 0                        ' \
    T1CONbits_TCKPS1 = 0                        ' / Timer1 Prescaler 1:1
    PR1 = cPR1_Value                            ' ADC Sampling rate
                                                ' Set the match cPR1_Value (will cause an interrupt when this value is reached)
    TMR1 = 0                                    ' Reset Timer1 value
    IFS0bits_T1IF = 0                           ' Clear the Timer1 interrupt flag
    IPC0bits_T1IP0 = 1                          ' \
    IPC0bits_T1IP1 = 1                          ' | High Priority Timer1 interrupt
    IPC0bits_T1IP2 = 1                          ' /
    T1CONbits_TON = 1                           ' Turn on Timer1
    IEC0bits_T1IE = 1                           ' Enable the Timer1 interrupt
'
'-----------------------------------------------
' Timer1 Interrupt handler
' Input     : None
' Output    : None
' Notes     : Triggers an ADC interrupt
'
Isr T1Interrupt
    If AD1CON1bits_DONE = 1 Then                ' If ADC is not busy
        AD1CON1bits_SAMP = 1                    ' Start new sample
    EndIf
    IFS0bits_T1IF = 0                           ' Clear the Timer1 interrupt flag
EndIsr
EndProc

'--------------------------------------------------------------------------------
' The main program starts here
'
Main:
    PLL_Setup(43, 2, 2, $0300)                  ' Configure the Oscillator to operate the device at 79.23MHz

    Global_wInIndex  = 0                        ' Reset the buffer index
    Clear Global_wInputBuf                      ' Clear the Global_wInputBuf array
'
' Initialise the Filter parameters with the buffer and coefficient details
'
    FIR_StructInit(FIRStruct, cCOEFF, cFILTER_TAPS, Global_wInputBuf, cBUFFER_SIZE)

    ADC_Start()                                 ' Initialise and start the ADC interrupt
    Timer1_Start()                              ' Initialise and start the Timer1 interrupt
    DAC1_Init(36, cDAC_Signed)                  ' Initialise the DAC for signed output
    DAC1L_Enable(0)                             ' Enable the DAC, no interrupts

    While                                       ' Create an infinite loop
        DAC1L_Write(Global_wFilterValue)        ' Write the filter value to the DAC
    Wend                                        ' Do it forever

'---------------------------------------------------------------------------------------
' Configure for internal 7.37MHz oscillator with PLL
' OSC pins are general purpose I/O
'
    Config FBS = BWRP_WRPROTECT_OFF
    Config FGS = GWRP_OFF
    Config FOSCSEL = FNOSC_FRCPLL, IESO_OFF
    Config FOSC = POSCMD_NONE, OSCIOFNC_ON, IOL1WAY_OFF, FCKSM_CSECME
    Config FWDT = WDTPOST_PS256, WINDIS_OFF, FWDTEN_OFF
    Config FPOR = FPWRT_PWR1, ALTI2C_OFF
    Config FICD = ICS_PGD1, JTAGEN_OFF

